/* PumpControl
 *
 *
 *  
 * 
 * 
 * Settings that you should configure
 * Remember - these settings are set after a startup or software restart.
 * 
 * ssid - Your WiFi SSID name
 * password - Your WiFi password
 * localUdpPort - UDP communication port number
 * IPAddress ip - ###.###.###.###
 * IPAddress gateway - ###.###.###.###
 * IPAddress subnet - ###.###.###.###
 * IPAddress dns - ###.###.###.###
 * servoStart - Potentiometer left stop
 * servoEnd - Potentiometer right stop
 * servoOffset - minimum flow rate at which the engine is still running well | The value can also be set via smartphone app.
 * Token - Your personal key to operate the controller; only with this correct key will received data processed
 * RemoteOnlyOne - true = only one IP adr can send commands to controller / false = any IP adr may send commands ... 
 * ShowError - true = Show Info & Errors on Client / false = no Infos and Errors
 * 
 * !!!Caution!!! Do not change any other variables, methods or functions in the code - the function is then no longer guaranteed.
 */



#include <WiFiUdp.h>
#include "ESP8266WiFi.h"
#include <Servo.h>
#include <FS.h>


Servo servoMain; // Define Servo Object
WiFiUDP Udp;     // Define WiFi - WLAN Object

const String AppInfo = "V1.11 | Date: 23.07.2022 | ML_SOFT";

const char * ssid = "MaltiFant2" ;                                                          // SSID eingeben; Enter the SSID-name of your WLAN
const char * password = "JJj#1aYZtoKrfBDb=ZqeG-jQ+#36%F@cX1&-AAf5Hc71UsbNq&C:7qYoGIbwa3";   // Passwort eingeben; Enter passwort


const String hostname = "ESP8266-01";       // Hostname; Enter host name
const byte servoIOpin = 0;                  // Output Servo Drive - this must not be changed
const byte onoffIOpin = 2;                  // Output ON/OFF Relay - this must not be changed
const byte soundIOpin = 1;                  // Output Sound - You can hear when your WiFi connection is OK. - this must not be changed
const byte pumpOn = 0; 
const byte pumpOff = 1; 
const boolean RemoteOnlyOne = true;         //true = only one Client / false = any Clients
const boolean ShowError = true;             //true --> Show Info & Errors on Client / false --> no Infos and Errors



const char* filename = "MyParam.txt";       //Filename on Filesystem
char incomingPacket[255];                   // Empfangsbuffer der eingehenden Datenpakete; buffer for incoming packets
char replyPacekt[] = "Received Data";       // Sende Empfangsquittierung; a reply string to send back
unsigned int localUdpPort = 8082;           // Lauschen auf dieser lokalen Portnummer; local port to listen on

                                            
int servoStart = 173;                       // Potentiometer Linksanschlag; Potentiometer left stop; Notes: Changes possible via smartphone app
int servoEnd = 6;                           // Potentiometer Rechtsanschlag; Potentiometer right stop; Notes: Changes possible via smartphone app 
int servoOffset = -70;                      // Mindestlaufleistung  - Mindest Fördermenge, bei der der Motor noch gut läuft; Minimum mileage - minimum flow rate at which the engine is still running well

byte powerMin = 50;                         // lowest output with pump on (xx%)
byte powerMax = 100;                        // highest pumping capacity - usually 100%
byte durationMin = 1;                       // Duration time from [minutes]
byte durationMax = 20;                      // Duration time to [minutes]
int reactionTime = 80;                      // smaller value - faster; greater value - slower
String Token = "123abc";                    // Token key | your secret key

// internal variables - this must not be changed
int pumpPower = 0;
long counter = 0;
boolean msg = true;
boolean firstStart = true;
String ReceiveData = "";
String rdOnOff;
int rdPower;
int rdPowerLast;
int rdTime;


// Send Variable
String sendString = "";
char sendArray[255] = "";
byte sendbackIP[4];

String strUdpremoteIP = "";
String strUdpremotePort = "";
long lngrssi = 0;
word paramErr = 0;



void setup() 
{
    Serial.begin (115200);
    
    pinMode(onoffIOpin, OUTPUT);
    pinMode(soundIOpin, INPUT);
    digitalWrite(onoffIOpin, pumpOff);    // Contactor OFF
    playSound(3);
    servoMain.attach(servoIOpin, 544, 2400);  // Servo IO-PIN Zuordnung      
    servoMain.write(servoStart);              // Niedrigste Leistung einstellen
    
    WiFi.persistent(false);                                              
    WiFi.mode(WIFI_OFF);
    WiFi.mode( WIFI_STA ); 

    IPAddress ip(192, 168, 10, 223);
    IPAddress gateway(192, 168, 10, 1);
    IPAddress subnet(255, 255, 255, 0);
    IPAddress dns(192, 168, 10, 10);
    WiFi.config(ip, dns, gateway, subnet);
    WiFi.hostname(hostname);
      
    
    // Connect to WiFi
    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) 
    {
      playSound(1);
      delay(500);
    }
    
    playSound(2);
    Udp.begin(localUdpPort); 

    WiFi.setAutoReconnect(true);
    WiFi.persistent(true);
}

void loop() 
{
  lngrssi = WiFi.RSSI(); 
  int packetSize = Udp.parsePacket();
  int i = 0;  

  if (packetSize)
  {

    int len = Udp.read(incomingPacket, 255);
    if (len > 0)
    {
      incomingPacket[len] = 0;
    }

    // Data reception and Special functions
    ReceiveData = incomingPacket;
      
    // Here continue with the pump control
    // Assign parameter data
    if (!readReceivedData()) goto NoAction;
    
        // send back a reply, to the IP address and port we got the packet from
        strUdpremotePort = String(Udp.remotePort());
        if (strUdpremotePort == String(localUdpPort))
        {
            if (strUdpremoteIP == "")
            {
              strUdpremoteIP = Udp.remoteIP().toString();  
              WriteUDP("Set Client-Remote IP: " + strUdpremoteIP); 
              delay(2000);
            }
            else
            {
                if (!RemoteOnlyOne)
                {
                  strUdpremoteIP = Udp.remoteIP().toString();  
                }
              
                if (strUdpremoteIP == Udp.remoteIP().toString())
                {
                    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
                    Udp.write(replyPacekt);
                    Udp.endPacket();                 
                }
                else
                {
                  if (RemoteOnlyOne && ShowError)
                  {
                    WriteUDP("Another client would like to take over control."); 
                    delay(2000);
                    WriteUDP("The Client-IP is: " + Udp.remoteIP().toString());
                    delay(2000);
                  }
                  goto NoAction;   
                }
            }   
        }
        else
        {
          goto NoAction;  
        }
       
        // Data reception and Special functions
        if (checkAppInfo1()) goto NoAction; //Send Controller Info 1 to client;  Showed version and network informations
        if (checkAppInfo2()) goto NoAction; //Send Controller Info 2 to client; Showed version and controller infos
        if (checkRestart()) goto NoAction;  //Controller restart; 
        
        //Parameter data plausibility check
        if (checkReceiveData())
        {
            if (rdOnOff == "ON")
            {
                if (firstStart)
                {
                    AddCounterFS();
                    playSound(4);
                    digitalWrite(onoffIOpin, pumpOn);  // Contactor ON; Power ON
                    // The pump motor should only start up at maximum Power, .... then reduce to the selected Power.
                    for (i = powerMax; i >= rdPower; i--) 
                    {
                        WriteUDP("New Start... Adjust Power -" + String(i));
                        servoMain.write(map(i, powerMin, powerMax, servoStart + servoOffset, servoEnd));
                        delay(reactionTime);
                    }
                    rdPowerLast =  rdPower; 
                    firstStart = false;  
                }

            // Change performance in operation
            changePowerUp();
            changePowerDown();
    
            // Cycle loop time 333 ms / Three runs per second
            counter = rdTime * 180;     
            }
            else
            {
                servoMain.write(servoStart);
                playSound(3);
                digitalWrite(onoffIOpin, pumpOff);    // Contactor OFF; Power OFF
                WriteUDP("Pump drive OFF");
                counter = 0;
                firstStart = true;
            }
        }
        else 
        {   
            if (ShowError)
            {
              WriteUDP("A parameter value is out of range!");
              playSound(5);
              delay(2000);
            }
            goto NoAction;  
        } // end of checkReceiveData()

    msg = true; 

  } //end of if (packetSize)
  
  //-------------------------------------------------------------------------------------
    if(counter <= 0)
    {
        servoMain.write(servoStart);   
        digitalWrite(onoffIOpin, pumpOff);    // Contactor OFF; Power OFF
        if (msg)
        {
            playSound(3);
            WriteUDP("Pump drive: OFF");
            msg = false;
            firstStart = true;
        }
    }
    else
    {
        counter = counter -1; 
        // Send Data back 
        WriteUDP("Countdown: " + ConvertTime(counter / 3) + " [Min:Sec]");        
    }
  //-------------------------------------------------------------------------------------
  NoAction: 

    // Send error messages to client if configured.
    if (ShowError && paramErr != 0)
    {
      sendErrorMsg();
    }
    
    // Cycle loop time 333 ms / three times a second
    // In this way, the receipt of new data can be processed quickly
    delay(333);  
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------


/* -----------------------------------------------------------------------------
 *  Functions and procedure
   -----------------------------------------------------------------------------*/


/* *****************************************************************************
 * sendErrorMsg to Client
 * parameters: no
 * Result: no
 * Date: 20.07.2022 Name: Lothar Maltenberger
 * 
 */
void sendErrorMsg()
{
      switch (paramErr) 
      {
        case 0b1:
          WriteUDP("Error Value 1 (on, off,...)");
          delay(777);
          break;
        case 0b10:
          WriteUDP("Error Value 2 (Pump Power)");
          delay(777);
          break;
        case 0b100:
          WriteUDP("Error Value 3 (Time)");
          delay(777);
          break;
        case 0b1000:
          WriteUDP("Error Value 4 (Set Power min)");
          delay(777);
          break;
        case 0b10000:
          WriteUDP("Error Value 5 (Set Power max)");
          delay(777);
          break;
        case 0b100000:
          WriteUDP("Error Value 6 (Runtime min)");
          delay(777);
          break;
        case 0b1000000:
          WriteUDP("Error Value 7 (Runtime max)");
          delay(777);
          break;
        case 0b10000000:
          WriteUDP("Error Value 8 (Servo Offset)");
          delay(777);
          break;
        case 0b100000000:
          WriteUDP("Error Value 9 (Servo speed)");
          delay(777);
          break;
        case 0b1000000000:
          WriteUDP("Error Value 10 (Token)");
          delay(777);
          break;
                                                                                
        default:
          WriteUDP("Unknown error occurred");
      }
  paramErr = 0;
  
}


/* *****************************************************************************
 * function readReceivedData and set to global variable
 * parameters: no
 * Result Value [byte] Errors 
 * Date: 10.06.2020 Name: Lothar Maltenberger
 * 
 */
bool readReceivedData()
{
    paramErr = 0;
    if (splitString(ReceiveData,';',0) != "-")
      rdOnOff = splitString(ReceiveData,';',0);
    else
      paramErr = 0b1;
    
    if (splitString(ReceiveData,';',1) != "-")
    {
      rdPowerLast = rdPower;
      rdPower = splitString(ReceiveData,';',1).toInt();
    }
    else
      paramErr = 0b10;
      
    if (splitString(ReceiveData,';',2) != "-")
      rdTime = splitString(ReceiveData,';',2).toInt();
    else
      paramErr = 0b100;

    if (splitString(ReceiveData,';',3) != "-")
      powerMin = splitString(ReceiveData,';',3).toInt();
    else
      paramErr = 0b1000;

    if (splitString(ReceiveData,';',4) != "-")
      powerMax = splitString(ReceiveData,';',4).toInt();
    else
      paramErr = 0b10000;

    if (splitString(ReceiveData,';',5) != "-")
      durationMin = splitString(ReceiveData,';',5).toInt();
    else
      paramErr = 0b100000;    

    if (splitString(ReceiveData,';',6)!= "-")
      durationMax = splitString(ReceiveData,';',6).toInt();
    else
      paramErr = 0b1000000;

    if (splitString(ReceiveData,';',7) != "-")
      servoOffset = splitString(ReceiveData,';',7).toInt();
    else
      paramErr = 0b10000000;    
    
    if (splitString(ReceiveData,';',8) != "-")
      reactionTime = splitString(ReceiveData,';',8).toInt();
    else
      paramErr = 0b100000000; 

    if (Token != splitString(ReceiveData,';',9))
      paramErr = 0b1000000000; 

    if (paramErr == 0)
      return true;
    else
      return false;
}



/* *****************************************************************************
 * function checkAppInfo1
 * parameters: no
 * Result Value [boolean]
 * Date: 10.06.2020 Name: Lothar Maltenberger
 * 
 */
bool checkAppInfo1()
{
    if (splitString(ReceiveData,';',0) == "APPINFO1")
    {
        if (Token == splitString(ReceiveData,';',9))
        {
            WriteAppInfo1();
            ReceiveData = "";
            return true;
        }
        else 
        {
            WriteUDP("Token wrong - AppInfo 1 not allowed");
            delay(2000);
            ReceiveData = "";
            return true;
        }
    }
    else 
    {
        return false;
    }
}



/* *****************************************************************************
 * function checkAppInfo2
 * parameters: no
 * Result Value [boolean]
 * Date: 10.06.2020 Name: Lothar Maltenberger
 * 
 */
bool checkAppInfo2()
{
    if (splitString(ReceiveData,';',0) == "APPINFO2")
    {
        if (Token == splitString(ReceiveData,';',9))
        {
            WriteAppInfo2();
            ReceiveData = "";
            return true;
        }
        else 
        {
            WriteUDP("Token wrong - AppInfo 2 not allowed");
            delay(2000);
            ReceiveData = "";
            return true;
        }
    }
    else 
    {
        return false;
    }
}




/* *****************************************************************************
 * function checkRestart
 * Übergabeparameter: no
 * Result Value [boolean]
 * Date: 10.06.2020 Name: Lothar Maltenberger
 * 
 */
bool checkRestart()
{
    if (splitString(ReceiveData,';',0) == "RESTART") 
    {
        if (Token == splitString(ReceiveData,';',9))
        {
            WriteUDP("Execution Controller-Restart now");
            delay(300);
            softwareRestart();
            ReceiveData = "";
            return true;
        }
        else 
        {
            WriteUDP("Token wrong - Controller-Restart not allowed");
            delay(3000);
            ReceiveData = "";
            return true;
        }
    }
    else 
    {
        return false;
    }
}




/* *****************************************************************************
 * function Convert integer-seconds to minutes and seconds
 * Parameters: intSeconds [int]
 * Result Value [String] Format ##:##
 * Date: 06.06.2020 Name: Lothar Maltenberger
 * Example: ConvertTime(180) --> 03:00 [minutes:seconds]
 */

String ConvertTime(int intSeconds) 
{
    String result;
    if (intSeconds / 60 < 10)
        result = "0" + String(intSeconds / 60);
    else
        result = String(intSeconds / 60);
    
    if (intSeconds % 60 < 10)
        result = result + ":0" + String(intSeconds % 60);
    else
        result = result + ":" + String(intSeconds % 60);

    return result;
}




/* *****************************************************************************
 * Function splitString
 * parameters: StringStream, Separator, Index
 * Result Value [String]
 * Date: 15.05.2020 Name: from Internet http://wp.scalesoft.de/arduino-split/ or https://forum.arduino.cc/index.php?topic=620138.0
 * Example: splitString(strS, ';',0)  | Index not found ... Result ---> "-"
 */
String splitString(String s, char parser, int index) 
{
    String rs="";
    int parserIndex = index;
    int parserCnt=0;
    int rFromIndex=0, rToIndex=-1;
    while (index >= parserCnt) 
    {
        rFromIndex = rToIndex+1;
        rToIndex = s.indexOf(parser,rFromIndex);
        if (index == parserCnt) 
        {
          if (rToIndex == 0 || rToIndex == -1) return "-";
          return s.substring(rFromIndex,rToIndex);
        }
        else 
          parserCnt++;
    }
    return rs;
}


/* *****************************************************************************
 * Function PlausiCheck ReceiveData Variables
 * GlobalVariables: rdOnOff, rdPower; rdTime, ect...
 * Result Value [boolean]
 * Date: 07.06.2020 Name: Lothar Maltenberger
 */
bool checkReceiveData() 
{
    if ((rdOnOff == "ON" || rdOnOff == "OFF") && 
        (rdPower >= powerMin) && (rdPower <= powerMax) && 
        (rdTime >= durationMin) && (rdTime <= durationMax) &&
        (powerMin >= 0) && (powerMin < powerMax) && (powerMax <= 100) && 
        (reactionTime >= 0) && (reactionTime <= 300) && 
        (servoStart >= 0) && (servoStart <= 180) &&
        (servoEnd >= 0) && (servoEnd <= 180) && 
        (servoOffset >= -180) && (servoOffset <= 180))
        return true; 
    else 
        return false;
}



/* *****************************************************************************
 * Function WriteUDP (send back a String to the IP address and port we got the packet from)
 * Variable: TextString [String]
 * Result Value [boolean]
 * Date: 30.05.2020 Name: Lothar Maltenberger
 */
bool WriteUDP(String TextString)
{   
    TextString.toCharArray(sendArray, 255);
    if(RemoteOnlyOne)
    {  
        sendbackIP[0] = splitString(strUdpremoteIP + '.','.',0).toInt();
        sendbackIP[1] = splitString(strUdpremoteIP + '.','.',1).toInt();
        sendbackIP[2] = splitString(strUdpremoteIP + '.','.',2).toInt();
        sendbackIP[3] = splitString(strUdpremoteIP + '.','.',3).toInt();
        Udp.beginPacket(sendbackIP, strUdpremotePort.toInt());
    }
    else
    {
        Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());    
    }
    Udp.write(sendArray);
    Udp.endPacket();
    memset(sendArray, '\0', sizeof(sendArray));
    return true; 
}


/* *****************************************************************************
 * Subroutine WriteAppInfo1 to Client
 * Variables: global
 * No results
 * Date: 01.06.2020 Name: Lothar Maltenberger
 */
void WriteAppInfo1()
{
    int dur = 1000;
    WriteUDP(AppInfo);
    delay(dur);
    WriteUDP("Contr. IP: " + WiFi.localIP().toString());
    delay(dur);
    WriteUDP("Contr. Subnet: " + WiFi.subnetMask().toString());
    delay(dur);
    WriteUDP("Contr. Gateway: " + WiFi.gatewayIP().toString());
    delay(dur);
    WriteUDP("Contr. DNS: " + WiFi.dnsIP().toString());
    delay(dur);
    if (RemoteOnlyOne)
      WriteUDP("Value RemoteOnlyOne = ON");
    else
      WriteUDP("Value RemoteOnlyOne = OFF");
    
    delay(dur);      
    
    if (ShowError)
      WriteUDP("Value ShowError = ON");
    else
      WriteUDP("Value ShowError = OFF");
    
    delay(dur);      
    counter = counter - 3 * 7 * 2;
}


/* *****************************************************************************
 * Subroutine WriteAppInfo2 to Client
 * Variables: global
 * No results
 * Date: 01.06.2020 Name: Lothar Maltenberger
 */
void WriteAppInfo2()
{
    int dur = 1000;
    WriteUDP(AppInfo);
    delay(dur);
    WriteUDP("WiFi SSID: " + String(ssid));
    delay(dur);
    WriteUDP("WiFi RSSI: " + String(lngrssi) + " db");
    delay(dur);
    WriteUDP("Client Remote IP: " + strUdpremoteIP);
    delay(dur);
    WriteUDP("Contr. Remote Port: " + strUdpremotePort);
    delay(dur);
    WriteUDP("Contr. Hostname: " + WiFi.hostname());
    delay(dur);
    WriteUDP("Power-On cycles: " + ReadCounterFS());
    delay(dur);
    counter = counter - 3 * 7 * 2;
}





/* *****************************************************************************
 * Subroutine Software Restart
 * Variables:non
 * Non results
 * Date: 01.06.2020 Name: Lothar Maltenberger
 */
void softwareRestart() 
{
    playSound(3);
    ESP.restart(); 
}



/* *****************************************************************************
 * Subroutine Sound
 * Variable: smode
 * Non results
 * Date: 01.06.2020 Name: Lothar Maltenberger
 */
void playSound(byte smode)
{
    pinMode(soundIOpin, OUTPUT);
    noTone(soundIOpin);

    switch (smode) 
    {
    case 1: //- - -
        delay(10);
        tone(soundIOpin, 700);
        delay(20);
        noTone(soundIOpin);
        pinMode(soundIOpin, INPUT);
        break;
    case 2: //----  ----  ----
        delay(1000);
        for (int i = 1; i <= 3; i++) 
        {
            tone(soundIOpin, 700);
            delay(500);
            noTone(soundIOpin);
            delay(500);
        }
        pinMode(soundIOpin, INPUT);
        break;
    case 3: // ---___
        delay(300);
        tone(soundIOpin, 900);
        delay(200);
        tone(soundIOpin, 700);
        delay(200);
        noTone(soundIOpin);
        pinMode(soundIOpin, INPUT);
        break;
    
    case 4: // ___---
        delay(300);
        tone(soundIOpin, 700);
        delay(200);
        tone(soundIOpin, 900);
        delay(200);
        noTone(soundIOpin);
        pinMode(soundIOpin, INPUT);
        break;
  case 5: // -
        delay(300);
        tone(soundIOpin, 900);
        delay(200);
        noTone(soundIOpin);
        pinMode(soundIOpin, INPUT);
        break;    

    }
    
        noTone(soundIOpin);
        pinMode(soundIOpin, INPUT);
}




/* *****************************************************************************
 * Subroutine Power change up... (Servo drive)
 * Variable: non
 * Non results
 * V1.09 Date: 01.06.2020 Name: Lothar Maltenberger
 * V1.11 Date: 17.06.2022 Name: Lothar Maltenberger
 */
void changePowerUp()
{
    int diff = 0; 
    while(rdPower > rdPowerLast) 
    {
        diff = rdPower - rdPowerLast;
        WriteUDP("Power change up... +" + String(diff));
        pumpPower = map(rdPowerLast, powerMin, powerMax, servoStart + servoOffset, servoEnd);
        servoMain.write(pumpPower);  
        rdPowerLast++;
        delay(reactionTime);
    }
}




/* *****************************************************************************
 * Subroutine Power change down... (Servo drive)
 * Variable: non
 * Non results
 * V1.09 Date: 01.06.2020 Name: Lothar Maltenberger
 * V1.11 Date: 17.06.2022 Name: Lothar Maltenberger
 */
void changePowerDown()
{
      int diff = 0; 
      while(rdPower < rdPowerLast) 
      {
          diff = rdPower - rdPowerLast;
          WriteUDP("Power change down... " + String(diff));
          pumpPower = map(rdPowerLast, powerMin, powerMax, servoStart + servoOffset, servoEnd);
          servoMain.write(pumpPower);  
          rdPowerLast--;
          delay(reactionTime);
      }
}







/* *****************************************************************************
 * Subroutine AddCounterFileSystem AddCounterFS()
 * Description: Create and Increase the counter on filesystem
 * Variable: non
 * Non results
 * V1.11 Date: 23.07.2022 Name: Lothar Maltenberger
 */
void AddCounterFS() 
{

  if(!SPIFFS.begin())
  {
    WriteUDP("Dateisystem: Fehler beim initialisieren");
    playSound(3);
    delay(1000);
    //SPIFFS.format();
    //SPIFFS.remove(filename);
  }
 
  File file = SPIFFS.open(filename, "r");
  if(!file)
  {
    // file not exists, create new file
    File file = SPIFFS.open(filename, "w");
    file.print("0");
    file.close();
  }
  
  if(file)
  {
     //file exists
     int i;
     String strCounter = "";
     for(i=0;i < file.size();i++)
     {
      strCounter = strCounter + (char)file.read();
     }

     long x = strCounter.toInt();
     x++;
     strCounter = String(x);
     File file = SPIFFS.open(filename, "w");
     file.print(strCounter);
     file.close();
  }
  
}




/* *****************************************************************************
 * Subroutine ReadCounterFS() 
 * Description: Read counter value from filesystem
 * Variable: non
 * Return: CounterValue as string
 * V1.11 Date: 23.07.2022 Name: Lothar Maltenberger
 */
String ReadCounterFS() 
{

  if(!SPIFFS.begin())
  {
    playSound(3);
    //SPIFFS.format();
    //SPIFFS.remove(filename);
    return "Init: ReadCounterFS()";
  }
 
  File file = SPIFFS.open(filename, "r");
  if(file)
  {
     //file exists
     int i;
     String strCounter = "";
     for(i=0;i < file.size();i++)
     {
      strCounter = strCounter + (char)file.read();
     }
     file.close();
     return strCounter;
  }
  else
  {
    return "0";   
  }
  
}





   
